#include "led.h"

//======================================================================//
//   RGB LED CHARLIEPLEXING LOGIK                                       //
//======================================================================//

// RGB Led Matrix
volatile RGBLed*	leds[LED_COUNT];
// Zhler fr die PWM Steuerung der Leds
volatile uint8_t	pwm_counter = 0;

//======================================================================//
//   Anlegen der LED Matrix - jede Led hat eine Anode und eine Kathode  //
//======================================================================//
void createMatrix()
{
	leds[ 0] = newRGBLed(newLed(1, 2), newLed(1, 3), newLed(1, 4));
	leds[ 1] = newRGBLed(newLed(5, 3), newLed(5, 2), newLed(5, 1));
	leds[ 2] = newRGBLed(newLed(1, 5), newLed(1, 6), newLed(1, 7));
	leds[ 3] = newRGBLed(newLed(5, 7), newLed(5, 6), newLed(5, 4));
	leds[ 4] = newRGBLed(newLed(2, 1), newLed(2, 3), newLed(2, 4));
	leds[ 5] = newRGBLed(newLed(6, 3), newLed(6, 2), newLed(6, 1));
	leds[ 6] = newRGBLed(newLed(2, 5), newLed(2, 6), newLed(2, 7));
	leds[ 7] = newRGBLed(newLed(6, 7), newLed(6, 5), newLed(6, 4));
	leds[ 8] = newRGBLed(newLed(3, 1), newLed(3, 2), newLed(3, 4));
	leds[ 9] = newRGBLed(newLed(7, 3), newLed(7, 2), newLed(7, 1));
	leds[10] = newRGBLed(newLed(3, 5), newLed(3, 6), newLed(3, 7));
	leds[11] = newRGBLed(newLed(7, 6), newLed(7, 5), newLed(7, 4));
	leds[12] = newRGBLed(newLed(4, 1), newLed(4, 2), newLed(4, 3));
	leds[13] = newRGBLed(newLed(8, 3), newLed(8, 2), newLed(8, 1));
	leds[14] = newRGBLed(newLed(4, 5), newLed(4, 6), newLed(4, 7));
	leds[15] = newRGBLed(newLed(8, 6), newLed(8, 5), newLed(8, 4));
}

//======================================================================//
//   Konstruktor zum Anlegen einer neuen Led                            //
//======================================================================//
Led* newLed(int anode, int kathode)
{
	Led* l = malloc(sizeof(Led));
	l->anode = anode;
	l->kathode = kathode;
	return l;
}

//======================================================================//
//   Konstruktor zum Anlegen einer neuen RGBLed                         //
//   Eine RGB Led speichert 3 Leds sowie deren PWM Wert (0 - 64)		//
//   Zudem speichert sie einen mode welcher ein oder aus sein kann		//
//======================================================================//
RGBLed* newRGBLed(Led *r, Led *g, Led *b)
{
	RGBLed* rgb = malloc(sizeof(RGBLed));
	rgb->mode = false;
	rgb->r_led = *r;
	rgb->g_led = *g;
	rgb->b_led = *b;
	rgb->r_pwm = 64;
	rgb->g_pwm = 64;
	rgb->b_pwm = 64;
	return rgb;
}

//======================================================================//
//   berprft den PWM Zhler mit den PWM Werten der Leds               //
//	 Je niedriger der PWM Wert, desto strker leuchtet die Led			//
//======================================================================//
void checkLedPWM()
{
	if (pwm_counter++ > 63)
	{
		resetRegisters();
		pwm_counter = 0;
	}
	for (int i = 0; i < LED_COUNT; i++) {
		if (leds[i]->mode && pwm_counter > leds[i]->r_pwm) turnOnLed(leds[i]->r_led);
		if (leds[i]->mode && pwm_counter > leds[i]->g_pwm) turnOnLed(leds[i]->g_led);
		if (leds[i]->mode && pwm_counter > leds[i]->b_pwm) turnOnLed(leds[i]->b_led);
	}
}

//======================================================================//
//   Schaltet die Register fr eine Led		                            //
//======================================================================//
void turnOnLed(Led led)
{
	resetRegisters();
	if (led.kathode < 7) {
		DDRC |= (1<<(led.kathode-1));
		} else {
		DDRD |= (1<<(led.kathode-4));
	}
	if (led.anode < 7) {
		DDRC |= (1<<(led.anode-1));
		PORTC |= (1<<(led.anode-1));
		} else {
		DDRD |= (1<<(led.anode-4));
		PORTD |= (1<<(led.anode-4));
	}
}

//======================================================================//
//   Methode zum aktivieren und deaktivieren einer RGB Led              //
//======================================================================//
void setLedMode(int led, bool on)
{
	leds[led]->mode = on;
}

//======================================================================//
//   Methode zum setzen der Farbwerte einer RGB Led                     //
//======================================================================//
void setLedColor(int led, uint8_t r, uint8_t g, uint8_t b)
{
	leds[led]->r_pwm = r;
	leds[led]->g_pwm = g;
	leds[led]->b_pwm = b;
}

//======================================================================//
//   Test-Methode zum aktivieren aller RGB Leds                         //
//======================================================================//
void turnOnLeds()
{
	for (int i = 0; i < LED_COUNT; i++) {
		setLedMode(i, true);
		setLedColor(i, 0, 0, 0);
	}
}

//======================================================================//
//   Methode zum gemeinsamen Deaktivieren aller RGB Leds                //
//======================================================================//
void turnOffLeds()
{
	for (int i = 0; i < LED_COUNT; i++) {
		setLedMode(i, false);
	}
}

//======================================================================//
//   Methode zum Anzeigen der ueren Begrenzung der Uhr                //
//======================================================================//
void turnOnOuterRing()
{
	setLedMode(15, true);
}

//======================================================================//
//   Test-Methode zum durchlaufen aller Mglichen Farbkombinationen     //
//   Zeigt einen flssigen Farbverlauf auf allen RGB Leds an			//
//======================================================================//
void loopColors()
{
	for (int i = 0; i < LED_COUNT; i++) {
		setLedMode(i, true);
	}
	for (char a = 0; a<65; a++)	{
		for (int i = 0; i < LED_COUNT; i++) {
			setLedColor(i, a, 64, 64-a);
		}
		for (long b = 0;b<100;b++){asm("nop");};
	}

	for (char a = 0; a<65; a++)	{
		for (int i = 0; i < LED_COUNT; i++) {
			setLedColor(i, 64, 64-a, a);
		}
		for (long b = 0;b<100;b++){asm("nop");};
	}
	
	for (char a = 0; a<65; a++)	{
		for (int i = 0; i < LED_COUNT; i++) {
			setLedMode(i, true);
			setLedColor(i, 64-a, a, 64);
		}
		for (long b = 0;b<100;b++){asm("nop");};
	}
}