#include "main.h"
#include "led.h"
#include "digital.h"
#include "analog.h"
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include "uart.h"

/************************************************************************/
/* Variablen                                                            */
/************************************************************************/

volatile uint8_t	segmentsPosition = 180;
volatile uint16_t	counter = 0;
volatile uint16_t	cmpValue = 0;
volatile uint16_t	lastInterrupt	= 0;
volatile uint8_t	overflowCnt		= 0;
volatile uint8_t	overflowTimer0Cnt = 0;
volatile uint8_t	timeFactorForSeconds = 0;
volatile uint8_t	seconds	= 0;
volatile uint8_t	minutes	= 20;
volatile uint8_t	hours	= 9;
volatile uint8_t	cntDigital = 0;
volatile bool		isDigitialClockEnabled = false;
volatile bool		isPacManStyleEnabled = false;
volatile bool		isPacManEating = false;

//Variablen zum Speichern in den EEPROM
extern uint16_t eep_1		EEMEM;
extern uint16_t eep_2		EEMEM;

/************************************************************************/
/* Main                                                                 */
/************************************************************************/
int main (void)
{
	init();
	
	// Zwischenspeicher fr Bluetooth Befehl
	char inputChar = UART_NO_DATA;

	// temporre Variablen
	int tempHours = 0;
	int tempMinutes = 0;
	int tempRColor = 0;
	int tempGColor = 0;
	int tempBColor = 0;
	
	// Counter zum mitzhlen der Stunden/Minuten-Stellen
	int stellenCnt = 0;
	int colorCnt = 0;
	bool hoursEnabled = false;
	bool minutesEnabled = false;
	bool rColorEnabled = false;
	bool gColorEnabled = false;
	bool bColorEnabled = false;
	
	// Setze Zeigerfarbe standardmig auf grn
	for (int i = 0; i < LED_COUNT-2; i++) {
		setLedColor(i, 64, 0, 64);
	}
	
	// Setze Segment- und Ring-Farbe auf blau
	setLedColor(14, 64, 64, 0);
	setLedColor(15, 64, 64, 0);
	
	// Betriebsschleife
	while(1) {
		// Zeige ueren Ring an
		turnOnOuterRing();
		
		// Nichts unternehmen wenn nichts empfangen wurde
		if( (inputChar = uart_getc()) == UART_NO_DATA) {
			continue;
		}
		
		// Zur Befehlsidentifizierung verwendete Chars:
		// s, m fr Empfangen der Zeit
		// r, g, b fr Empfangen der Zeiger-Farbe
		// t, f fr Empfangen des DigitalUhrModus
		// p, a fr Empfangen des PacManModus
		
		//-----Stunden ------------------------------------
		// 's' als Stunden-Char wurde empfangen
		if (inputChar == 's') {
			hoursEnabled = true;
			stellenCnt++;
			// Debug-Ausgaben
			uart_puts("Hours Enabled");
		}
		// Erste Stelle wurde empfangen
		else if(hoursEnabled == true && stellenCnt == 1) {
			// ASCII-Wert in Dezimalzahl umrechnen...
			// ... dann mit 10 multiplizieren, ( da dieses Zeichen das erste Zeichen der Stunden ist )
			// ... dann zwischenspeichern
			// ASCII-Wert 1 = 0x31 in Dezimal -> Deshalb "- 0x30"
			tempHours = ( inputChar - 0x30) * 10;
			stellenCnt++;
			// Debug-Ausgaben
			uart_puts("ErsteStelle:");
			uart_putc(tempHours);
		}
		// Zweite Stelle wurde empfangen und setzen der Uhr
		else if (hoursEnabled == true && stellenCnt == 2){
			// in Dezimalzahl umrechnen + ersteStundenStelle
			hours = (inputChar - 0x30 ) + tempHours;
			hoursEnabled=false;
			uart_puts("Stunden:");
			uart_putc(hours);
			// Zaehler zuruecksetzen
			stellenCnt = 0;
		}
		//----- Minuten ------------------------------------
		// 'm' als Minuten-Char wurde empfangen
		if (inputChar == 'm') {
			minutesEnabled = true;
			stellenCnt++;
			uart_puts("Minutes Enabled");
		}
		// Erste Stelle wurde emfangen
		else if(minutesEnabled == true && stellenCnt == 1) {
			stellenCnt++;
			tempMinutes = ( inputChar - 0x30) * 10;
			uart_puts("ErsteStelle:");
			uart_putc(tempMinutes);
		}
		// Zweite Stelle wurde emfangen und setzen der Uhr
		else if (minutesEnabled == true && stellenCnt == 2) {
			minutes = (inputChar - 0x30 ) + tempMinutes;
			minutesEnabled=false;
			
			uart_puts("Minutes:");
			uart_putc(minutes);
			// Zaehler zuruecksetzen
			stellenCnt = 0;
		}
		
		//-----R Farb-Wert ------------------------------------
		// 'r' als Rot-Char wurde empfangen
		if (inputChar == 'r') {
			rColorEnabled = true;
			colorCnt++;
			// Debug-Ausgaben
			uart_puts("R Color Enabled");
		}
		// Erste Stelle wurde empfangen
		else if(rColorEnabled == true && stellenCnt == 1) {
			// ASCII-Wert in Dezimalzahl umrechnen...
			// ... dann mit 10 multiplizieren, ( da dieses Zeichen das erste Zeichen der Stunden ist )
			// ... dann zwischenspeichern
			// ASCII-Wert 1 = 0x31 in Dezimal -> Deshalb "- 0x30"
			tempRColor = ( inputChar - 0x30) * 10;
			colorCnt++;
			// Debug-Ausgaben
			uart_puts("R-FarbWert:");
			uart_putc(tempRColor);
		}
		// Zweite Stelle wurde empfangen und setzen der Farbe
		else if (rColorEnabled == true && colorCnt == 2){
			// in Dezimalzahl umrechnen
			int rColor = (inputChar - 0x30 ) + tempRColor;
			for (int i = 0; i < 14; i++) {
				leds[i]->r_pwm = rColor;
			}
			rColorEnabled = false;
			uart_puts("R-Farbe:");
			uart_putc(rColor);
			// Zhler zurcksetzen
			colorCnt = 0;
		}
		//-----G Farb-Wert ------------------------------------
		// 'g' als Grn-Char wurde empfangen
		if (inputChar == 'g') {
			gColorEnabled = true;
			colorCnt++;
			uart_puts("G Color Enabled");
		}
		// Erste Stelle wurde emfangen
		else if(gColorEnabled == true && colorCnt == 1) {
			colorCnt++;
			tempGColor = ( inputChar - 0x30) * 10;
			uart_puts("G-FarbWert:");
			uart_putc(tempGColor);
		}
		// Zweite Stelle wurde emfangen und setzen der Farbe
		else if (gColorEnabled == true && colorCnt == 2) {
			int gColor = (inputChar - 0x30 ) + tempGColor;
			for (int i = 0; i < 14; i++) {
				leds[i]->g_pwm = gColor;
			}
			gColorEnabled = false;
			
			uart_puts("G-Farbe:");
			uart_putc(gColor);
			// Zaehler zuruecksetzen
			colorCnt = 0;
		}
		//-----B Farb-Wert ------------------------------------
		// 'b' als Blau-Char wurde empfangen
		if (inputChar == 'b') {
			bColorEnabled = true;
			colorCnt++;
			uart_puts("B Color Enabled");
		}
		// Erste Stelle wurde emfangen
		else if(bColorEnabled == true && colorCnt == 1) {
			colorCnt++;
			tempBColor = ( inputChar - 0x30) * 10;
			uart_puts("B-FarbWert:");
			uart_putc(tempBColor);
		}
		// Zweite Stelle wurde emfangen und setzen der Farbe
		else if (bColorEnabled == true && colorCnt == 2) {
			int bColor = (inputChar - 0x30 ) + tempBColor;
			for (int i = 0; i < 14; i++) {
				leds[i]->b_pwm = bColor;
			}
			bColorEnabled = false;
			uart_puts("B-Farbe:");
			uart_putc(bColor);
			// Zaehler zuruecksetzen
			colorCnt = 0;
		}
		
		if (inputChar == 't') {
			// DigitalUhr-Modus aktivieren
			isDigitialClockEnabled = true;
			setLedMode(0, true);
			} else if (inputChar == 'f') {
			// DigitalUhr-Modus deaktivieren
			isDigitialClockEnabled = false;
			setLedMode(0, false);
			} else if (inputChar == 'p') {
			// PacMan-Modus aktivieren
			isPacManStyleEnabled = true;
			setLedMode(1, true);
			} else if (inputChar == 'a') {
			// PacMan-Modus deaktivieren
			isPacManStyleEnabled = false;
			setLedMode(1, false);
		}
	}
}

/************************************************************************/
/* Initialisierungsmethode                                              */
/************************************************************************/
void init() {
	board_init();
	resetRegisters();	
	createMatrix();
	uart_init(UART_BAUD_SELECT(BAUD, F_CPU));
	
	//TIMSK0 |= (1 << TOIE0);
	//TCCR0B |= (1<<CS00 | 0<<CS01 | 0<<CS02);
	// Port B auf Ausgang setzen
	DDRB = 0xff;
	// Port C auf Ausgang setzen
	DDRC  = 0xff;
	// Port D auf Ausgang setzen
	DDRD = 0xff;
	// Lichtschranke PD2 auf Eingang
	DDRD = DDRD & ~(1<<PD2);

	// LEDs auf den Ports ausschalten
	PORTB = 0xff;
	PORTC = 0xff;
	PORTD = 0xff;
	
	EIMSK |= _BV(INT0);		// PD2 = INT0 enablen
	EICRA |= _BV(ISC01);	// Triggern auf fallende Flanke des INT0

	//Timer1 fuer die Umlaeufe konfigurieren
	TCCR1B = (1 << CS10) | (1 << CS21) ;	//kein Prescaler
	TIMSK1 = (1 << TOIE1) | (1 << OCIE1A) ;
	
	TIMSK2 |= (1 << TOIE2);
	TCCR2B |= (1<<CS20 | 0<<CS21 | 0<<CS22);

	// Timer0 fuer die Stundenanzahl konfigurieren
	// CompareMatchInterupt ausloesen
	// Timer0 fuer die Uhrzeiten-Logik und Overflow fuer Timer0
	TIMSK0 = _BV(OCIE0A) | _BV(TOIE0);	// Enable Interrupt TimerCounter0 Compare Match A (SIG_OUTPUT_COMPARE0A)
	TCCR0A = _BV(WGM01);				// Mode = CTC
	TCCR0B = _BV(CS02) | _BV(CS00);		// Clock/1024, 0.001024 seconds per tick
	OCR0A = 244;						// 0.001024*244 ~= .25 SIG_OUTPUT_COMPARE0A will be triggered 4 times per second.
	resetRegisters();
	// Das Makro sei() schaltet die Interrupts ein.
	sei();
}

void resetRegisters() {
	// Port D auf Ausgang setzen
	DDRD = 0b11100111;
	// Lichtschranke PD2 auf Eingang
	DDRD = DDRD & ~(1<<PD2);
	
	DDRC = 0x00;
	PORTC = 0x00;
	PORTD = 0x00;
}

/************************************************************************/
/*  Interrupt                                                           */
/************************************************************************/
ISR(TIMER0_OVF_vect)
{
	setLedMode(1,true);
	overflowTimer0Cnt++;
	if (overflowTimer0Cnt == 7) {
		setTime();
		overflowTimer0Cnt = 0;
	}
}

/************************************************************************/
/*  Compare-Interrupt                                                   */
/************************************************************************/
ISR(TIMER0_COMPA_vect)
{
	// hochzhlen des Compare-Takts
	timeFactorForSeconds++;
	// Viertel-Sekunden-Takt deshalb jede 14-ste mal rein. Da es dann exakt eine Sekunde ist.
	if(timeFactorForSeconds >= 14) {
		timeFactorForSeconds = 0;
		turnOffLeds();
		setTime(); // Zeit-Funktion aufrufen
	}
}

/************************************************************************/
/*  TIMER1 Overflow Interrupt... berlufe zhlen                       */
/************************************************************************/
ISR(TIMER1_OVF_vect)
{
	overflowCnt++;
}

/************************************************************************/
/*  Lichtschranken-Interrupt                                            */
/*  Hier Timer starten fr Counter der Umlufe und der Geschwindigkeit. */
/************************************************************************/
ISR (INT0_vect)
{
	turnOffLeds();
	_delay_ms(5000);
	//Die segmentsPosition wird nach einem Durchlauf zurueck auf 180 gesetzt
	segmentsPosition = 180;
	
	//Wert des counters
	counter = TCNT1;

	// Falls ein berlauf des Timer1 stattfindet
	if(overflowCnt > 0) {
		cmpValue = (counter / 180) + (overflowCnt *  65535);
		overflowCnt = 0;
		} else {
		// Vergleichswert errechnen, damit die Timer1_Compa_vect auf gerufen wird
		// In diesem Fall 180mal bei jedem Durchlauf
		cmpValue = counter / 180;
	}
	// Vergleichswert in Laufvariable speichern
	lastInterrupt = cmpValue;
	// Vergleichswert setzen, danach wird Timer1_Compa_vect stetig aufgerufen
	OCR1A = cmpValue;
	// Timer zurcksetzen
	TCNT1 = 0;
}

/************************************************************************/
/* Timer1_Compa_vect                                                    */
/************************************************************************/
ISR (TIMER1_COMPA_vect)
{
	// Laufvariable mit dem letzten Interrupt um Vergleichstwert erhoehen
	lastInterrupt += cmpValue;
	// Vergleichswert neu setzen. Naechster Vergleichswert wird gesetzt
	OCR1A = lastInterrupt;
	
	// Alles LEDs ausschalten
	turnOffLeds();
	
	// Falls Digitaluhr-anzeige aktiviert wurde
	if (isDigitialClockEnabled) {
		showDigitalTime();
	}
	
	//Stundenpositionen anzeigen
	if (segmentsPosition % 15 == 0 && segmentsPosition < 180) {
		
		setLedMode(14, true);
		// Pacman-punkte anzeigen
		if (isPacManStyleEnabled == true) {
			setLedMode(13, true);
		}
	}
	
	// Variablen zum Vergleichen der Segmentenposition (180) und der Zeiteinheiten (60sekunden, 60 Minuten und 12 Stunden)
	uint8_t sec = seconds * 3;
	uint8_t min = minutes * 3;
	uint8_t ho = hours * 3 * 5;
	
	// fr SekundenZeiger
	if(segmentsPosition == sec && isPacManStyleEnabled == false) {
		displaySecondsHand();
	}
	// Pacman Methode aufrufen
	if (isPacManStyleEnabled) {
		enablePacManStyle();
	}

	// Minutenzeiger mit Pfeil	
	if (segmentsPosition == min - 1 || segmentsPosition == min + 1) {
		setLedMode(14, true);
	} else if (segmentsPosition == min){
		displayMinutesHand();
	}
	
	// Stundenzeiger	
	if (segmentsPosition == ho) {
		displayHoursHand();
	}

	// zuruecksetzen beim Maxwert
	if(segmentsPosition == 0) {
		segmentsPosition = 180;
		//TCNT1 = 0;
	} else {
		segmentsPosition--;
	}
}

ISR (TIMER2_OVF_vect)
{
	checkLedPWM();
}

/************************************************************************/
/*  Zeitmethode                                                         */
/************************************************************************/
void setTime(void)
{
	if(seconds >= 60) {
		minutes++;
		seconds = 0;
	}
	if(minutes >= 60) {
		hours++;
		minutes = 0;
	}
	if(hours >= 12) {
		hours = 0;
	}
	seconds++;
}

void enablePacManStyle(void)
{
	turnOffLeds();
	
	int sec = seconds *3;
	if(isPacManEating) {
		// LED10	-----#-----
		// LED09	---#####---
		// LED08	-######----
		// LED07	####-------
		// LED06	-###-------
		// LED05	--#######--
		// LED04	----###----
		if (segmentsPosition == sec-10) {
			setLedMode(6, true);
		} else
		if (segmentsPosition == sec-8) {
			setLedMode(7, true); setLedMode(6, true); setLedMode(5, true);
		} else
		if (segmentsPosition == sec-6) {
			setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true);
		} else
		if (segmentsPosition == sec-4) {
			setLedMode(8, true); setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true);
		} else
		if (segmentsPosition == sec-2) {
			setLedMode(8, true); setLedMode(7, true); setLedMode(4, true); setLedMode(3, true);
		} else
		if (segmentsPosition == sec) {
			setLedMode(9, true); setLedMode(8, true); setLedMode(7, true); setLedMode(4, true); setLedMode(3, true);
		} else
		if (segmentsPosition == sec+2 ) {
			setLedMode(8, true); setLedMode(7, true); setLedMode(4, true); setLedMode(3, true);
		} else
		if (segmentsPosition == sec+4) {
			setLedMode(8, true); setLedMode(4, true);
		} else
		if (segmentsPosition == sec+6) {
			setLedMode(4, true);
		}
		} else {
		// LED10	-----#-----
		// LED09	---#####---
		// LED08	-#########-
		// LED07	###########
		// LED06	-#########-
		// LED05	--#######--
		// LED04	----###----
		if(segmentsPosition == sec-10 || segmentsPosition == sec+10) {
			setLedMode(6, true);
		} else
		if(segmentsPosition == sec-8 || segmentsPosition == sec+8) {
			setLedMode(7, true); setLedMode(6, true); setLedMode(5, true);
		} else
		if(segmentsPosition == sec-6 || segmentsPosition == sec+6) {
			setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true);
		} else
		if(segmentsPosition == sec-4 || segmentsPosition == sec+4) {
			setLedMode(8, true); setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true);
		} else
		if(segmentsPosition == sec-2 || segmentsPosition == sec+2) {
			setLedMode(8, true); setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true); setLedMode(3, true);
		} else
		if(segmentsPosition == sec) {
			setLedMode(9, true); setLedMode(8, true); setLedMode(7, true); setLedMode(6, true); setLedMode(5, true); setLedMode(4, true); setLedMode(3, true);
		}
	}
	// nur an diesen Positionen darf Pacman fressen :)
	isPacManEating = (seconds % 2 == 1 && seconds < 60);
}